<?php
// $Id: template.inc,v 1.1 2009/12/16 21:25:13 dvessel Exp $


/**
 * PRE-PRE-PROCESS -> PRE-PROCESS -> RE-PROCESS (variable processors)
 * ---------------------------------------------------------------------------
 * This theme allows three distinct phases of variable processors to work.
 * Drupal core normally allows only one phase named "preprocess". Hexagon allows
 * the following:
 *  - pre_preprocess      - Early phase.
 *  - preprocess          - Normally allowed in core. Will be invoked as usual.
 *  - re_process          - Late phase.
 * 
 * Then there is the implementation level indicated by the prefix.
 *  - template            - Normally the owner of the hook implements this.
 *  - MODULE_NAME         - Module tapping into the hook. Not the owner.
 *  - ENGINE_NAME_engine  - Implemented by the theme engine itself. Not used.
 *  - ENGINE_NAME         - Engine name used by the theme. Not recommended.
 *  - THEME_NAME          - Theme implementation.
 *  - SUBTHEME_NAME       - Sub-theme implementation.
 *  - PLUGIN_NAME         - Plugin implementation.
 * 
 * "page" hook example in order of execution:
 *  - hex_pre_preprocess
 *  - template_preprocess
 *  - template_preprocess_page
 *  - hex_re_process
 * 
 * @see http://drupal.org/node/223430
 *  How preprocess functions normally work.
 * 
 * @see http://drupal.org/node/306358
 *  Drupal 7 includes the second phase variable processor but they are named
 *  "process" not "re_process" as it's done here in the late phase. Process is
 *  too commonly used for other functions and conflicts may occur. It will be
 *  safe to change when the theme is upgraded for 7.
 * 
 */


/**
 * These variables will be available to all templates, everywhere!
 */
function hex_pre_preprocess(&$vars, $hook) {
  static $hooks;

  if (!isset($hooks[$hook])) {
    $registry = theme_get_registry();
    // Include dependent files.
    if (!empty($registry[$hook]['includes'])) {
      foreach ($registry[$hook]['includes'] as $include_path) {
        include_once $include_path;
      }
    }
    // Useful registry data made available to each template.
    $hooks[$hook]['template_hook'] = $hook;
    foreach ($registry[$hook] as $key => $data) {
      $hooks[$hook]['template_' . str_replace(' ', '_', $key)] = $data;
    }
    // Safeguard against fatal errors. Make sure the functions exist if this is populated.
    if (!empty($hooks[$hook]['template_post_render_functions'])) {
      foreach ($hooks[$hook]['template_post_render_functions'] as $post_render) {
        if (!function_exists($post_render)) {
          unset($hooks[$hook]['template_post_render_functions'][$post_render]);
        }
      }
    }
  }
  foreach ($hooks[$hook] as $key => $data) {
    $vars[$key] = $data;
  }
  // Link references to variable keys that originate from the theme() function call.
  foreach (array_keys($vars['template_arguments']) as $parameter) {
    $vars['template_arguments'][$parameter] = &$vars[$parameter];
  }

  // Initialize id and class attributes.
  if (!isset($vars['id_attribute'])) {
    $vars['id_attribute'] = '';
  }
  $vars['classes_array'][] = $hook;
  // Reference variables so they can be added either way.
  $vars['html_elements']['#attributes']['id']    = &$vars['id_attribute'];
  $vars['html_elements']['#attributes']['class'] = &$vars['classes_array'];
}

/**
 * Third phase variable processor for all theming hooks implemented as templates.
 * 
 * Variable process functions that run after this should not add any further
 * template naming suggestions.
 */
function hex_re_process(&$vars, $hook) {
  // Hook added for consistency.
  $template_files[$hook] = $hook;
  // Add in clean version to minimize confusion.
  $template_files[hex_clean_string($hook)] = hex_clean_string($hook);
  foreach ($vars['template_files'] as $template) {
    $template_files[$template] = $template;
    $template_files[hex_clean_string($template)] = hex_clean_string($template);
  }
  $vars['template_files'] = array_values($template_files);
}

/**
 * Render a system default template, which is essentially a PHP template.
 *
 * Overridden to allow a final point of interception by themes and plug-ins.
 * Will not work on templates owned by modules. 
 *
 * @param $template_file
 *   The filename of the template to render. Note that this will overwrite
 *   anything stored in $variables['template_file'] if using a preprocess hook.
 * @param $variables
 *   A keyed array of variables that will appear in the output.
 *
 * @return
 *   The output generated by the template.
 */
function phptemplate_render_template($template_file, $variables) {
  extract($variables, EXTR_SKIP + EXTR_REFS);  // Extract the variables to a local namespace
  ob_start();                                  // Start output buffering
  include "./$template_file";                  // Include the template file
  $contents = ob_get_contents();               // Get the contents of the buffer
  ob_end_clean();                              // End buffering and discard
  _hex_post_render($variables, $contents);     // Invoke post renderer.
  return $contents;                            // Return final content.
}

/**
 * Invokes post processing when available.
 */
function _hex_post_render(&$vars, &$contents) {
  if (!empty($vars['template_post_render_functions'])) {
    $vars['post_content'] = $contents;
    foreach ($vars['template_post_render_functions'] as $post_render) {
      $post_render($vars, $vars['template_hook']);
      $vars['post_content_array'][$post_render] = $vars['post_content'];
    }
    $contents = $vars['post_content'];
  }
}